# T.Ottmann, P.Widmayer: Algorithmen und Datenstrukturen, BI, 1990
# page 108

proc MergeSort {} {
    global field

    MergeSort1 0 [expr $field(len) - 1]
}

proc MergeSort1 {l r} {

    if {$l < $r} {
	set m [expr ($l + $r) / 2]
	MergeSort1 $l            $m
	MergeSort1 [expr $m + 1] $r

	Merge $l $m $r
    }
}

proc Merge {l m r} {
    global field

    # puts "Merge $l $m $r"

    set i $l
    set j [expr $m + 1]
    set k $l

    while {$i <= $m && $j <= $r} {
	# select minimum value of heads of sequences

	if {[compare $field($i) <= $field($j)]} {
	    set tmp($k) $field($i)
	    incr i
	} else {
	    set tmp($k) $field($j)
	    incr j
	}

	incr k
    }

    if {$i > $m} {
	# 1. sequence empty, copy rest of 2. sequence

	for {set h $j} {$h <= $r} {incr h} {
	    set tmp([expr $k + $h - $j]) $field($h)
	}
    } else {
	# 2. sequence empty, copy rest of 1. sequence

	for {set h $i} {$h <= $m} {incr h} {
	    set tmp([expr $k + $h - $i]) $field($h)
	}

    }

    # copy temporary field into original field

    for {set h $l} {$h <= $r} {incr h} {
	set field($h) $tmp($h)
    }
}

# T.Ottmann, P.Widmayer: Algorithmen und Datenstrukturen, BI, 1990
# page 111

proc StraightMergeSort {} {
    global field

    set l 0
    set r [expr $field(len) - 1]

    set size 1

    while {$size < $r - $l + 1} {

	set rr [expr $l - 1]
	while {$rr + $size < $r} {

	    set ll [expr $rr + 1]
	    set mm [expr $ll + $size - 1]
	    if {$mm + $size <= $r} {
		set rr [expr $mm + $size]
	    } else {
		set rr $r
	    }

	    Merge $ll $mm $rr
	}
	incr size $size
    }

}

# T.Ottmann, P.Widmayer: Algorithmen und Datenstrukturen, BI, 1990
# page 113

proc NaturalMergeSort {} {
    global field

    set l 0
    set r [expr $field(len) - 1]

    set ll [expr $r + 1]

    while {$ll != $l} {
	set rr [expr $l - 1]

	while {$rr < $r} {
	    set ll  [expr $rr + 1]

	    set mm  $ll
	    while { $mm < $r && [compare $field([expr $mm + 1]) >= $field($mm)] } {
		incr mm
	    }

	    if {$mm < $r} {

		set rr [expr $mm + 1]
		while {$rr < $r && [compare $field([expr $rr + 1]) >= $field($rr)]} {
		    incr rr
		}

		Merge $ll $mm $rr
	    } else {
		set rr $mm
	    }
	}
    }
}